@@ -94,6 +94,9 @@
 #'   (`character`, `character`) -> `self` \cr
 #'   Rename [`PipeOp`]s: Change ID of each [`PipeOp`] as identified by `old` to the corresponding item in `new`. This should be used
 #'   instead of changing a [`PipeOp`]'s `$id` value directly!
+#' * `update_ids(prefix = "", postfix = "")` \cr
+#'   (`character`, `character`) -> `self` \cr
+#'   Pre- or postfix [`PipeOp`]'s existing ids. Both `prefix` and `postfix` default to `""`, i.e. no changes.
 #' * `train(input, single_input = TRUE)` \cr
 #'   (`any`, `logical(1)`) -> named `list`\cr
 #'   Train [`Graph`] by traversing the [`Graph`]s' edges and calling all the [`PipeOp`]'s `$train` methods in turn.
@@ -299,14 +302,17 @@ Graph = R6Class("Graph",
           return(txt)
         })
         ig_data$nodes$title = paste0("<p>", ig_data$nodes$title, "</p>")
-        ig_data$edges$color = "lightblue"
+        edges = NROW(ig_data$edges)
+        if (edges) ig_data$edges$color = "lightblue"  # Only if more than one pipeop
         # Visualize the nodes
         p = visNetwork::visNetwork(nodes = ig_data$nodes, edges = ig_data$edges, height = "400px", width = "50%")
-        p = visNetwork::visIgraphLayout(p, layout = "layout_with_sugiyama", type = "full") 
-
-        # Draw edges between points
-        p = visNetwork::visEdges(p, arrows = "to", smooth = list(enabled = FALSE, forceDirection = "vertical"))
-        p 
+        if (edges) { # Only if more than one pipeop
+          # Set up layout
+          p = visNetwork::visIgraphLayout(p, layout = "layout_with_sugiyama", type = "full")
+          # Draw edges between points
+          p = visNetwork::visEdges(p, arrows = "to", smooth = list(enabled = FALSE, forceDirection = "vertical"))
+        }
+        p
       } else {
         suppressWarnings(plot(ig, layout = layout))  # suppress partial matching warning
       }
@@ -352,6 +358,11 @@ Graph = R6Class("Graph",
       self$edges[, c("src_id", "dst_id") := list(map_values(src_id, old, new), map_values(dst_id, old, new))]
       invisible(self)
     },
+    update_ids = function(prefix = "", postfix = "") {
+      ids = names2(self$pipeops)
+      self$set_names(ids, sprintf("%s%s%s", assert_string(prefix), ids, assert_string(postfix)))
+      invisible(self)
+    },
 
     train = function(input, single_input = TRUE) {
       graph_load_namespaces(self, "train")
@@ -465,7 +476,7 @@ graph_channels_dt = function(ids, channels, pipeops, direction) {
 # input: as given by `$train`, `$predict`. single valued to be copied (if
 #   `single_input` is `TRUE`) or (possibly named) list of values for each
 #   incoming edge.
-# fun: function of each `PipeOp` to call; should be `train` oder
+# fun: function of each `PipeOp` to call; should be `train` or
 #   `predict`.
 # single_input: whether `input` is to be copied to all input channels
 #   (`TRUE`) or is a list with different input for each channel (`FALSE`).
@@ -479,6 +490,17 @@ graph_reduce = function(self, input, fun, single_input) {
   edges = copy(self$edges)
 
   # create virtual "__initial__" and "__terminal__" nodes with edges to inputs / outputs of graph.
+  # if we have `single_input == FALSE` and one(!) vararg channel, we widen the vararg input
+  # appropriately.
+  if (!single_input && length(assert_list(input)) > nrow(graph_input) && "..." %in% graph_input$channel.name) {
+    if (sum("..." == graph_input$channel.name) != 1) {
+      stop("Ambiguous distribution of inputs to vararg channels.\nAssigning more than one input to vararg channels when there are multiple vararg inputs does not work.")
+    }
+    # repeat the "..." as often as necessary
+    repeats = ifelse(graph_input$channel.name == "...", length(input) - nrow(graph_input) + 1, 1)
+    graph_input = graph_input[rep(graph_input$name, repeats), , on = "name"]
+  }
+
   edges = rbind(edges,
     data.table(src_id = "__initial__", src_channel = graph_input$name,
       dst_id = graph_input$op.id, dst_channel = graph_input$channel.name),
@@ -489,11 +511,14 @@ graph_reduce = function(self, input, fun, single_input) {
   edges$payload = list()
 
   if (!single_input) {
+    # we need the input list length to be equal to the number of channels. This number was
+    # already increased appropriately if there is a single vararg channel.
+    assert_list(input, len = nrow(graph_input))
     # input can be a named list (will be distributed to respective edges) or unnamed.
     # if it is named, we check that names are unambiguous.
-    assert_list(input, len = nrow(graph_input))
     if (!is.null(names(input))) {
       if (anyDuplicated(graph_input$name)) {
+        # FIXME this will unfortunately trigger if there is more than one named input for a vararg channel.
         stopf("'input' must not be a named list because Graph %s input channels have duplicated names.", self$id)
       }
       assert_names(names(input), subset.of = graph_input$name)
@@ -517,6 +542,8 @@ graph_reduce = function(self, input, fun, single_input) {
     input = input_tbl$payload
     names(input) = input_tbl$name
 
+    lg$debug("Running PipeOp '%s$%s()'", id, fun, pipeop = op, input = input)
+
     output = op[[fun]](input)
     if (self$keep_results) {
       op$.result = output
