@@ -57,17 +57,6 @@ LearnerClassifAvg = R6Class("LearnerClassifAvg", inherit = LearnerClassif,
       )
       self$param_set$values = list(algorithm = "NLOPT_LN_COBYLA")
     },
-
-    train_internal = function(task) {
-      pars = self$param_set$get_values(tags = "train")
-      data = self$prepare_data(task)
-      n_weights = length(data)
-      list("weights" = optimize_objfun_nlopt(task, pars, self$weighted_average_prediction, n_weights, data))
-    },
-
-    predict_internal = function(task) {
-      self$weighted_average_prediction(task, self$model$weights, self$prepare_data(task))
-    },
     prepare_data = function(task) {
       data = task$data(cols = task$feature_names)
       fcts = map_lgl(data, is.factor)
@@ -99,7 +88,7 @@ LearnerClassifAvg = R6Class("LearnerClassifAvg", inherit = LearnerClassif,
       prob = NULL
       response = NULL
       if (self$predict_type == "response") {
-        response = weighted_factor_mean(data, weights, task$class_names)
+        response = factor(task$class_names[max.col(weighted_factor_mean(data, weights, task$class_names))], levels = task$class_names)  # ties broken at random
       } else {
         prob = weighted_matrix_sum(data, weights)
         prob = pmin(pmax(prob, 0), 1)
@@ -107,6 +96,17 @@ LearnerClassifAvg = R6Class("LearnerClassifAvg", inherit = LearnerClassif,
 
       PredictionClassif$new(row_ids = task$row_ids, truth = task$truth(), response = response, prob = prob)
     }
+  ),
+  private = list(
+    .train = function(task) {
+      pars = self$param_set$get_values(tags = "train")
+      data = self$prepare_data(task)
+      n_weights = length(data)
+      list("weights" = optimize_objfun_nlopt(task, pars, self$weighted_average_prediction, n_weights, data))
+    },
+    .predict = function(task) {
+      self$weighted_average_prediction(task, self$model$weights, self$prepare_data(task))
+    }
   )
 )
 
@@ -130,17 +130,6 @@ LearnerRegrAvg = R6Class("LearnerRegrAvg", inherit = LearnerRegr,
       )
       self$param_set$values = list(algorithm = "NLOPT_LN_COBYLA")
     },
-
-    train_internal = function(task) {
-      pars = self$param_set$get_values(tags = "train")
-      data = self$prepare_data(task)
-      n_weights = ncol(data$response_matrix)
-      list("weights" = optimize_objfun_nlopt(task, pars, self$weighted_average_prediction, n_weights, data))
-    },
-
-    predict_internal = function(task) {
-      self$weighted_average_prediction(task, self$model$weights, self$prepare_data(task))
-    },
     prepare_data = function(task) {
       response_matrix = as.matrix(task$data(cols = grep("\\.response$", task$feature_names, value = TRUE)))
       list(response_matrix = response_matrix)
@@ -152,6 +141,17 @@ LearnerRegrAvg = R6Class("LearnerRegrAvg", inherit = LearnerRegr,
       se = NULL
       PredictionRegr$new(row_ids = task$row_ids, truth = task$truth(), response = response, se = se)
     }
+  ),
+  private = list(
+    .train = function(task) {
+      pars = self$param_set$get_values(tags = "train")
+      data = self$prepare_data(task)
+      n_weights = ncol(data$response_matrix)
+      list("weights" = optimize_objfun_nlopt(task, pars, self$weighted_average_prediction, n_weights, data))
+    },
+    .predict = function(task) {
+      self$weighted_average_prediction(task, self$model$weights, self$prepare_data(task))
+    }
   )
 )
 
