@@ -47,8 +47,10 @@
 #'   instance count), `"major"`, `"minor"`, `"nonmajor"`, `"nonminor"` (see respective values
 #'   for `$reference`), `"upsample"` (only upsample), and `"downsample"`. Initialized to `"all"`.
 #' * `shuffle` :: `logical(1)` \cr
-#'   Whether to shuffle the result. Otherwise, the resulting task will have the original items that
-#'   were not removed in downsampling in-order, followed by all newly sampled items ordered by target class.
+#'   Whether to shuffle the rows of the resulting task.
+#'   In case the data is upsampled and `shuffle = FALSE`, the resulting task will have the original
+#'   rows (which were not removed in downsampling) in the original order, followed by all newly added rows
+#'   ordered by target class.
 #'   Initialized to `TRUE`.
 #'
 #' @section Internals:
@@ -110,10 +112,12 @@ PipeOpClassBalancing = R6Class("PipeOpClassBalancing",
         ParamLgl$new("shuffle", default = TRUE, tags = "train")
       ))
       ps$values = list(ratio = 1, reference = "all", adjust = "all", shuffle = TRUE)
-      super$initialize(id, param_set = ps, param_vals = param_vals, can_subset_cols = FALSE, task_type = "TaskClassif")
-    },
+      super$initialize(id, param_set = ps, param_vals = param_vals, can_subset_cols = FALSE, task_type = "TaskClassif", tags = "imbalanced data")
+    }
+  ),
+  private = list(
 
-    train_task = function(task) {
+    .train_task = function(task) {
 
       self$state = list()
       truth = task$truth()
@@ -156,7 +160,7 @@ PipeOpClassBalancing = R6Class("PipeOpClassBalancing",
       task_filter_ex(task, new_ids)
     },
 
-    predict_task = identity
+    .predict_task = identity
   )
 )
 
