@@ -30,7 +30,7 @@
 #' @section Parameters:
 #' The parameters are the parameters inherited from [`PipeOpTaskPreproc`], as well as:
 #' * `numsplits`  :: `numeric(1)` \cr
-#'   Number of bins to create. Default is \code{2}.
+#'   Number of bins to create. Default is `2`.
 #'
 #' @section Internals:
 #' Uses the [`stats::quantile`] function.
@@ -59,21 +59,19 @@ PipeOpQuantileBin = R6Class("PipeOpQuantileBin",
         ParamInt$new("numsplits",  default = 2, lower = 2, special_vals = list(NULL), tags = "train")
         ))
       ps$values = list(numsplits = 2L)
-      super$initialize(id, param_set = ps, param_vals = param_vals, packages = "stats")
-    },
+      super$initialize(id, param_set = ps, param_vals = param_vals, packages = "stats", feature_types = c("numeric", "integer"))
+    }
+  ),
+  private = list(
 
-    select_cols = function(task) {
-      task$feature_types[get("type") %in% c("numeric", "integer"), get("id")]
-    },
-
-    get_state_dt = function(dt, levels, target) {
+    .get_state_dt = function(dt, levels, target) {
       bins = lapply(dt, function(d)
         unique(c(-Inf, quantile(d, (1:(self$param_set$values$numsplits - 1)) /
             self$param_set$values$numsplits, na.rm = TRUE), Inf)))
       list(bins = bins)
     },
 
-    transform_dt = function(dt, levels) {
+    .transform_dt = function(dt, levels) {
       as.data.frame(mapply(function(d, b) ordered(cut(d, breaks = b)), d = dt,
         b = self$state$bins, SIMPLIFY = FALSE), row.names = rownames(dt))
     }
